# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2017-11-16 16:58
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix_email', '0004_auto_20171108_1628'),
    ]

    operations = [
        migrations.AddField(
            model_name='emailmessage',
            name='retries',
            field=models.PositiveIntegerField(default=0, verbose_name='Retries'),
        ),
    ]
