# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-09-21 10:06
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix_email', '0002_auto_20170502_1043'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='emailtemplatetexten',
            options={'default_permissions': ('add', 'change', 'delete', 'view', 'list')},
        ),
        migrations.AlterModelOptions(
            name='emailtemplatetextes',
            options={'default_permissions': ('add', 'change', 'delete', 'view', 'list')},
        ),
    ]
